# DmasApi

All URIs are relative to *https://astronomy.weathersourceapis.com/v2*

Method | HTTP request | Description
------------- | ------------- | -------------
[**dmasDmaIdDaysDateGet**](DmasApi.md#dmasDmaIdDaysDateGet) | **GET** /dmas/{dmaId}/days/{date} | A day of astronomy data for a DMA.
[**dmasDmaIdDaysDateStartdateEndGet**](DmasApi.md#dmasDmaIdDaysDateStartdateEndGet) | **GET** /dmas/{dmaId}/days/{dateStart},{dateEnd} | A contiguous range of daily astronomy for a DMA.
[**dmasDmaIdHoursTimestampGet**](DmasApi.md#dmasDmaIdHoursTimestampGet) | **GET** /dmas/{dmaId}/hours/{timestamp} | An hour of astronomy data for a DMA
[**dmasDmaIdHoursTimestampStarttimestampEndGet**](DmasApi.md#dmasDmaIdHoursTimestampStarttimestampEndGet) | **GET** /dmas/{dmaId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly astronomy for a DMA.

<a name="dmasDmaIdDaysDateGet"></a>
# **dmasDmaIdDaysDateGet**
> AstronomyDmaDayObj dmasDmaIdDaysDateGet(dmaId, date, fields, unitScale)

A day of astronomy data for a DMA.

Returns a single day of astronomical data for a Designated Market Area.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.DmasApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

DmasApi apiInstance = new DmasApi();
Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
LocalDate date = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyDmaDayObj result = apiInstance.dmasDmaIdDaysDateGet(dmaId, date, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling DmasApi#dmasDmaIdDaysDateGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **dmaId** | **Integer**| A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). |
 **date** | **LocalDate**| An date formatted as a RFC3339 date value. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyDmaDayObj**](AstronomyDmaDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="dmasDmaIdDaysDateStartdateEndGet"></a>
# **dmasDmaIdDaysDateStartdateEndGet**
> AstronomyDmaDayObj dmasDmaIdDaysDateStartdateEndGet(dmaId, dateStart, dateEnd, fields, unitScale)

A contiguous range of daily astronomy for a DMA.

Returns a contiguous range of daily astronomical data for a Designated Market Area.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.DmasApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

DmasApi apiInstance = new DmasApi();
Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
LocalDate dateStart = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
LocalDate dateEnd = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally after dateStart.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyDmaDayObj result = apiInstance.dmasDmaIdDaysDateStartdateEndGet(dmaId, dateStart, dateEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling DmasApi#dmasDmaIdDaysDateStartdateEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **dmaId** | **Integer**| A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). |
 **dateStart** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally before dateEnd. |
 **dateEnd** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally after dateStart. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyDmaDayObj**](AstronomyDmaDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="dmasDmaIdHoursTimestampGet"></a>
# **dmasDmaIdHoursTimestampGet**
> AstronomyDmaHourObj dmasDmaIdHoursTimestampGet(dmaId, timestamp, fields, unitScale)

An hour of astronomy data for a DMA

Returns a single hour of astronomical data for a Designated Market Area.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.DmasApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

DmasApi apiInstance = new DmasApi();
Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
OffsetDateTime timestamp = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyDmaHourObj result = apiInstance.dmasDmaIdHoursTimestampGet(dmaId, timestamp, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling DmasApi#dmasDmaIdHoursTimestampGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **dmaId** | **Integer**| A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). |
 **timestamp** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyDmaHourObj**](AstronomyDmaHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="dmasDmaIdHoursTimestampStarttimestampEndGet"></a>
# **dmasDmaIdHoursTimestampStarttimestampEndGet**
> AstronomyDmaHourObj dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId, timestampStart, timestampEnd, fields, unitScale)

A contiguous range of hourly astronomy for a DMA.

Returns a contiguous range of hourly astronomical data for a Designated Market Area.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.DmasApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

DmasApi apiInstance = new DmasApi();
Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
OffsetDateTime timestampStart = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
OffsetDateTime timestampEnd = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyDmaHourObj result = apiInstance.dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId, timestampStart, timestampEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling DmasApi#dmasDmaIdHoursTimestampStarttimestampEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **dmaId** | **Integer**| A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). |
 **timestampStart** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. |
 **timestampEnd** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyDmaHourObj**](AstronomyDmaHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

