# PointsApi

All URIs are relative to *https://astronomy.weathersourceapis.com/v2*

Method | HTTP request | Description
------------- | ------------- | -------------
[**pointsLatitudelongitudeDaysDateGet**](PointsApi.md#pointsLatitudelongitudeDaysDateGet) | **GET** /points/{latitude},{longitude}/days/{date} | A day of astronomy data for a Latitude/Longitude point.
[**pointsLatitudelongitudeDaysDateStartdateEndGet**](PointsApi.md#pointsLatitudelongitudeDaysDateStartdateEndGet) | **GET** /points/{latitude},{longitude}/days/{dateStart},{dateEnd} | A contiguous range of daily astronomy for a Latitude/Longitude point.
[**pointsLatitudelongitudeHoursTimestampGet**](PointsApi.md#pointsLatitudelongitudeHoursTimestampGet) | **GET** /points/{latitude},{longitude}/hours/{timestamp} | An hour of astronomy data for a Latitude/Longitude point.
[**pointsLatitudelongitudeHoursTimestampStarttimestampEndGet**](PointsApi.md#pointsLatitudelongitudeHoursTimestampStarttimestampEndGet) | **GET** /points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly astronomy for a Latitude/Longitude point.

<a name="pointsLatitudelongitudeDaysDateGet"></a>
# **pointsLatitudelongitudeDaysDateGet**
> AstronomyPointDayObj pointsLatitudelongitudeDaysDateGet(latitude, longitude, date, fields, unitScale)

A day of astronomy data for a Latitude/Longitude point.

Returns a single day of astronomical data for a Latitude/Longitude point.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PointsApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PointsApi apiInstance = new PointsApi();
Float latitude = 3.4F; // Float | A latitude value between -90° and 90°.
Float longitude = 3.4F; // Float | A longitude value between -180° (West) and 180° (East).
LocalDate date = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPointDayObj result = apiInstance.pointsLatitudelongitudeDaysDateGet(latitude, longitude, date, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PointsApi#pointsLatitudelongitudeDaysDateGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **latitude** | **Float**| A latitude value between -90° and 90°. |
 **longitude** | **Float**| A longitude value between -180° (West) and 180° (East). |
 **date** | **LocalDate**| An date formatted as a RFC3339 date value. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPointDayObj**](AstronomyPointDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="pointsLatitudelongitudeDaysDateStartdateEndGet"></a>
# **pointsLatitudelongitudeDaysDateStartdateEndGet**
> AstronomyPointDayObj pointsLatitudelongitudeDaysDateStartdateEndGet(latitude, longitude, dateStart, dateEnd, fields, unitScale)

A contiguous range of daily astronomy for a Latitude/Longitude point.

Returns a contiguous range of daily astronomical data for a Latitude/Longitude point.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PointsApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PointsApi apiInstance = new PointsApi();
Float latitude = 3.4F; // Float | A latitude value between -90° and 90°.
Float longitude = 3.4F; // Float | A longitude value between -180° (West) and 180° (East).
LocalDate dateStart = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
LocalDate dateEnd = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally after dateStart.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPointDayObj result = apiInstance.pointsLatitudelongitudeDaysDateStartdateEndGet(latitude, longitude, dateStart, dateEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PointsApi#pointsLatitudelongitudeDaysDateStartdateEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **latitude** | **Float**| A latitude value between -90° and 90°. |
 **longitude** | **Float**| A longitude value between -180° (West) and 180° (East). |
 **dateStart** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally before dateEnd. |
 **dateEnd** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally after dateStart. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPointDayObj**](AstronomyPointDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="pointsLatitudelongitudeHoursTimestampGet"></a>
# **pointsLatitudelongitudeHoursTimestampGet**
> AstronomyPointHourObj pointsLatitudelongitudeHoursTimestampGet(latitude, longitude, timestamp, fields, unitScale)

An hour of astronomy data for a Latitude/Longitude point.

Returns a single hour of astronomical data for a Latitude/Longitude point.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PointsApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PointsApi apiInstance = new PointsApi();
Float latitude = 3.4F; // Float | A latitude value between -90° and 90°.
Float longitude = 3.4F; // Float | A longitude value between -180° (West) and 180° (East).
OffsetDateTime timestamp = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPointHourObj result = apiInstance.pointsLatitudelongitudeHoursTimestampGet(latitude, longitude, timestamp, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PointsApi#pointsLatitudelongitudeHoursTimestampGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **latitude** | **Float**| A latitude value between -90° and 90°. |
 **longitude** | **Float**| A longitude value between -180° (West) and 180° (East). |
 **timestamp** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPointHourObj**](AstronomyPointHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="pointsLatitudelongitudeHoursTimestampStarttimestampEndGet"></a>
# **pointsLatitudelongitudeHoursTimestampStarttimestampEndGet**
> AstronomyPointHourObj pointsLatitudelongitudeHoursTimestampStarttimestampEndGet(latitude, longitude, timestampStart, timestampEnd, fields, unitScale)

A contiguous range of hourly astronomy for a Latitude/Longitude point.

Returns a contiguous range of hourly astronomical data for a Latitude/Longitude point.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PointsApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PointsApi apiInstance = new PointsApi();
Float latitude = 3.4F; // Float | A latitude value between -90° and 90°.
Float longitude = 3.4F; // Float | A longitude value between -180° (West) and 180° (East).
OffsetDateTime timestampStart = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
OffsetDateTime timestampEnd = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPointHourObj result = apiInstance.pointsLatitudelongitudeHoursTimestampStarttimestampEndGet(latitude, longitude, timestampStart, timestampEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PointsApi#pointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **latitude** | **Float**| A latitude value between -90° and 90°. |
 **longitude** | **Float**| A longitude value between -180° (West) and 180° (East). |
 **timestampStart** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. |
 **timestampEnd** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPointHourObj**](AstronomyPointHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

