# PostcodesApi

All URIs are relative to *https://astronomy.weathersourceapis.com/v2*

Method | HTTP request | Description
------------- | ------------- | -------------
[**postcodesPostcodecountryCodeDaysDateGet**](PostcodesApi.md#postcodesPostcodecountryCodeDaysDateGet) | **GET** /postcodes/{postcode},{countryCode}/days/{date} | A day of astronomy data for a postcode.
[**postcodesPostcodecountryCodeDaysDateStartdateEndGet**](PostcodesApi.md#postcodesPostcodecountryCodeDaysDateStartdateEndGet) | **GET** /postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd} | A contiguous range of daily astronomy for a postcode.
[**postcodesPostcodecountryCodeHoursTimestampGet**](PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestamp} | An hour of astronomy data for a postcode.
[**postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet**](PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly astronomy for a postcode.

<a name="postcodesPostcodecountryCodeDaysDateGet"></a>
# **postcodesPostcodecountryCodeDaysDateGet**
> AstronomyPostcodeDayObj postcodesPostcodecountryCodeDaysDateGet(postcode, countryCode, date, fields, unitScale)

A day of astronomy data for a postcode.

Returns a single day of astronomical data for a postcode.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PostcodesApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PostcodesApi apiInstance = new PostcodesApi();
String postcode = "postcode_example"; // String | A postcode string.
String countryCode = "countryCode_example"; // String | An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
LocalDate date = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPostcodeDayObj result = apiInstance.postcodesPostcodecountryCodeDaysDateGet(postcode, countryCode, date, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PostcodesApi#postcodesPostcodecountryCodeDaysDateGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **postcode** | **String**| A postcode string. |
 **countryCode** | **String**| An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). |
 **date** | **LocalDate**| An date formatted as a RFC3339 date value. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPostcodeDayObj**](AstronomyPostcodeDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="postcodesPostcodecountryCodeDaysDateStartdateEndGet"></a>
# **postcodesPostcodecountryCodeDaysDateStartdateEndGet**
> AstronomyPostcodeDayObj postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode, countryCode, dateStart, dateEnd, fields, unitScale)

A contiguous range of daily astronomy for a postcode.

Returns a contiguous range of daily astronomical data for a postcode.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PostcodesApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PostcodesApi apiInstance = new PostcodesApi();
String postcode = "postcode_example"; // String | A postcode string.
String countryCode = "countryCode_example"; // String | An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
LocalDate dateStart = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
LocalDate dateEnd = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally after dateStart.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPostcodeDayObj result = apiInstance.postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode, countryCode, dateStart, dateEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PostcodesApi#postcodesPostcodecountryCodeDaysDateStartdateEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **postcode** | **String**| A postcode string. |
 **countryCode** | **String**| An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). |
 **dateStart** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally before dateEnd. |
 **dateEnd** | **LocalDate**| An date formatted as a RFC3339 date value that must occur temporally after dateStart. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunTransit, sunrise, sunset, civilRise, civilSet, nauticalRise, nauticalSet, astroRise, astroSet, moonDistance, moonRadius, moonrise, moonset, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPostcodeDayObj**](AstronomyPostcodeDayObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="postcodesPostcodecountryCodeHoursTimestampGet"></a>
# **postcodesPostcodecountryCodeHoursTimestampGet**
> AstronomyPostcodeHourObj postcodesPostcodecountryCodeHoursTimestampGet(postcode, countryCode, timestamp, fields, unitScale)

An hour of astronomy data for a postcode.

Returns a single hour of astronomical data for a postcode.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PostcodesApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PostcodesApi apiInstance = new PostcodesApi();
String postcode = "postcode_example"; // String | A postcode string.
String countryCode = "countryCode_example"; // String | An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
OffsetDateTime timestamp = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPostcodeHourObj result = apiInstance.postcodesPostcodecountryCodeHoursTimestampGet(postcode, countryCode, timestamp, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PostcodesApi#postcodesPostcodecountryCodeHoursTimestampGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **postcode** | **String**| A postcode string. |
 **countryCode** | **String**| An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). |
 **timestamp** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPostcodeHourObj**](AstronomyPostcodeHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

<a name="postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet"></a>
# **postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet**
> AstronomyPostcodeHourObj postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale)

A contiguous range of hourly astronomy for a postcode.

Returns a contiguous range of hourly astronomical data for a postcode.

### Example
```java
// Import classes:
//import io.swagger.client.ApiClient;
//import io.swagger.client.ApiException;
//import io.swagger.client.Configuration;
//import io.swagger.client.auth.*;
//import io.swagger.client.api.PostcodesApi;

ApiClient defaultClient = Configuration.getDefaultApiClient();

// Configure API key authorization: ApiKeyAuth
ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
ApiKeyAuth.setApiKey("YOUR API KEY");
// Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
//ApiKeyAuth.setApiKeyPrefix("Token");

PostcodesApi apiInstance = new PostcodesApi();
String postcode = "postcode_example"; // String | A postcode string.
String countryCode = "countryCode_example"; // String | An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
OffsetDateTime timestampStart = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
OffsetDateTime timestampEnd = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)* 
String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
try {
    AstronomyPostcodeHourObj result = apiInstance.postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale);
    System.out.println(result);
} catch (ApiException e) {
    System.err.println("Exception when calling PostcodesApi#postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet");
    e.printStackTrace();
}
```

### Parameters

Name | Type | Description  | Notes
------------- | ------------- | ------------- | -------------
 **postcode** | **String**| A postcode string. |
 **countryCode** | **String**| An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). |
 **timestampStart** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. |
 **timestampEnd** | **OffsetDateTime**| A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. |
 **fields** | [**List&lt;String&gt;**](String.md)| A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  | [enum: sunDistance, sunRadius, sunAzimuth, sunElevation, moonDistance, moonRadius, moonAzimuth, moonElevation, moonPhase, moonIllum, moonAge, all, allMoon, allSun, popular]
 **unitScale** | **String**| The unit scale for returned values.&lt;br&gt; | [optional] [default to IMPERIAL] [enum: IMPERIAL, METRIC, SI]

### Return type

[**AstronomyPostcodeHourObj**](AstronomyPostcodeHourObj.md)

### Authorization

[ApiKeyAuth](../README.md#ApiKeyAuth)

### HTTP request headers

 - **Content-Type**: Not defined
 - **Accept**: application/json

