/*
 * Weather Source Astronomy API
 * Daily and hourly astronomical data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source Astronomy API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */

package io.swagger.client.api;

import io.swagger.client.ApiCallback;
import io.swagger.client.ApiClient;
import io.swagger.client.ApiException;
import io.swagger.client.ApiResponse;
import io.swagger.client.Configuration;
import io.swagger.client.Pair;
import io.swagger.client.ProgressRequestBody;
import io.swagger.client.ProgressResponseBody;

import com.google.gson.reflect.TypeToken;

import java.io.IOException;


import io.swagger.client.model.AstronomyDmaDayObj;
import io.swagger.client.model.AstronomyDmaHourObj;
import io.swagger.client.model.ErrorObj;
import org.threeten.bp.LocalDate;
import org.threeten.bp.OffsetDateTime;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DmasApi {
    private ApiClient apiClient;

    public DmasApi() {
        this(Configuration.getDefaultApiClient());
    }

    public DmasApi(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    public ApiClient getApiClient() {
        return apiClient;
    }

    public void setApiClient(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    /**
     * Build call for dmasDmaIdDaysDateGet
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdDaysDateGetCall(Integer dmaId, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/dmas/{dmaId}/days/{date}"
            .replaceAll("\\{" + "dmaId" + "\\}", apiClient.escapeString(dmaId.toString()))
            .replaceAll("\\{" + "date" + "\\}", apiClient.escapeString(date.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call dmasDmaIdDaysDateGetValidateBeforeCall(Integer dmaId, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'dmaId' is set
        if (dmaId == null) {
            throw new ApiException("Missing the required parameter 'dmaId' when calling dmasDmaIdDaysDateGet(Async)");
        }
        // verify the required parameter 'date' is set
        if (date == null) {
            throw new ApiException("Missing the required parameter 'date' when calling dmasDmaIdDaysDateGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling dmasDmaIdDaysDateGet(Async)");
        }
        
        com.squareup.okhttp.Call call = dmasDmaIdDaysDateGetCall(dmaId, date, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A day of astronomy data for a DMA.
     * Returns a single day of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyDmaDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyDmaDayObj dmasDmaIdDaysDateGet(Integer dmaId, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyDmaDayObj> resp = dmasDmaIdDaysDateGetWithHttpInfo(dmaId, date, fields, unitScale);
        return resp.getData();
    }

    /**
     * A day of astronomy data for a DMA.
     * Returns a single day of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyDmaDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyDmaDayObj> dmasDmaIdDaysDateGetWithHttpInfo(Integer dmaId, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = dmasDmaIdDaysDateGetValidateBeforeCall(dmaId, date, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyDmaDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A day of astronomy data for a DMA. (asynchronously)
     * Returns a single day of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdDaysDateGetAsync(Integer dmaId, LocalDate date, List<String> fields, String unitScale, final ApiCallback<AstronomyDmaDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = dmasDmaIdDaysDateGetValidateBeforeCall(dmaId, date, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyDmaDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for dmasDmaIdDaysDateStartdateEndGet
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdDaysDateStartdateEndGetCall(Integer dmaId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/dmas/{dmaId}/days/{dateStart},{dateEnd}"
            .replaceAll("\\{" + "dmaId" + "\\}", apiClient.escapeString(dmaId.toString()))
            .replaceAll("\\{" + "dateStart" + "\\}", apiClient.escapeString(dateStart.toString()))
            .replaceAll("\\{" + "dateEnd" + "\\}", apiClient.escapeString(dateEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call dmasDmaIdDaysDateStartdateEndGetValidateBeforeCall(Integer dmaId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'dmaId' is set
        if (dmaId == null) {
            throw new ApiException("Missing the required parameter 'dmaId' when calling dmasDmaIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateStart' is set
        if (dateStart == null) {
            throw new ApiException("Missing the required parameter 'dateStart' when calling dmasDmaIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateEnd' is set
        if (dateEnd == null) {
            throw new ApiException("Missing the required parameter 'dateEnd' when calling dmasDmaIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling dmasDmaIdDaysDateStartdateEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = dmasDmaIdDaysDateStartdateEndGetCall(dmaId, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of daily astronomy for a DMA.
     * Returns a contiguous range of daily astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyDmaDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyDmaDayObj dmasDmaIdDaysDateStartdateEndGet(Integer dmaId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyDmaDayObj> resp = dmasDmaIdDaysDateStartdateEndGetWithHttpInfo(dmaId, dateStart, dateEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of daily astronomy for a DMA.
     * Returns a contiguous range of daily astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyDmaDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyDmaDayObj> dmasDmaIdDaysDateStartdateEndGetWithHttpInfo(Integer dmaId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = dmasDmaIdDaysDateStartdateEndGetValidateBeforeCall(dmaId, dateStart, dateEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyDmaDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of daily astronomy for a DMA. (asynchronously)
     * Returns a contiguous range of daily astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdDaysDateStartdateEndGetAsync(Integer dmaId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyDmaDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = dmasDmaIdDaysDateStartdateEndGetValidateBeforeCall(dmaId, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyDmaDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for dmasDmaIdHoursTimestampGet
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdHoursTimestampGetCall(Integer dmaId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/dmas/{dmaId}/hours/{timestamp}"
            .replaceAll("\\{" + "dmaId" + "\\}", apiClient.escapeString(dmaId.toString()))
            .replaceAll("\\{" + "timestamp" + "\\}", apiClient.escapeString(timestamp.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call dmasDmaIdHoursTimestampGetValidateBeforeCall(Integer dmaId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'dmaId' is set
        if (dmaId == null) {
            throw new ApiException("Missing the required parameter 'dmaId' when calling dmasDmaIdHoursTimestampGet(Async)");
        }
        // verify the required parameter 'timestamp' is set
        if (timestamp == null) {
            throw new ApiException("Missing the required parameter 'timestamp' when calling dmasDmaIdHoursTimestampGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling dmasDmaIdHoursTimestampGet(Async)");
        }
        
        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampGetCall(dmaId, timestamp, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * An hour of astronomy data for a DMA
     * Returns a single hour of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyDmaHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyDmaHourObj dmasDmaIdHoursTimestampGet(Integer dmaId, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyDmaHourObj> resp = dmasDmaIdHoursTimestampGetWithHttpInfo(dmaId, timestamp, fields, unitScale);
        return resp.getData();
    }

    /**
     * An hour of astronomy data for a DMA
     * Returns a single hour of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyDmaHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyDmaHourObj> dmasDmaIdHoursTimestampGetWithHttpInfo(Integer dmaId, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampGetValidateBeforeCall(dmaId, timestamp, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyDmaHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * An hour of astronomy data for a DMA (asynchronously)
     * Returns a single hour of astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdHoursTimestampGetAsync(Integer dmaId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ApiCallback<AstronomyDmaHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampGetValidateBeforeCall(dmaId, timestamp, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyDmaHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for dmasDmaIdHoursTimestampStarttimestampEndGet
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdHoursTimestampStarttimestampEndGetCall(Integer dmaId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/dmas/{dmaId}/hours/{timestampStart},{timestampEnd}"
            .replaceAll("\\{" + "dmaId" + "\\}", apiClient.escapeString(dmaId.toString()))
            .replaceAll("\\{" + "timestampStart" + "\\}", apiClient.escapeString(timestampStart.toString()))
            .replaceAll("\\{" + "timestampEnd" + "\\}", apiClient.escapeString(timestampEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call dmasDmaIdHoursTimestampStarttimestampEndGetValidateBeforeCall(Integer dmaId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'dmaId' is set
        if (dmaId == null) {
            throw new ApiException("Missing the required parameter 'dmaId' when calling dmasDmaIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampStart' is set
        if (timestampStart == null) {
            throw new ApiException("Missing the required parameter 'timestampStart' when calling dmasDmaIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampEnd' is set
        if (timestampEnd == null) {
            throw new ApiException("Missing the required parameter 'timestampEnd' when calling dmasDmaIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling dmasDmaIdHoursTimestampStarttimestampEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampStarttimestampEndGetCall(dmaId, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of hourly astronomy for a DMA.
     * Returns a contiguous range of hourly astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyDmaHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyDmaHourObj dmasDmaIdHoursTimestampStarttimestampEndGet(Integer dmaId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyDmaHourObj> resp = dmasDmaIdHoursTimestampStarttimestampEndGetWithHttpInfo(dmaId, timestampStart, timestampEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of hourly astronomy for a DMA.
     * Returns a contiguous range of hourly astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyDmaHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyDmaHourObj> dmasDmaIdHoursTimestampStarttimestampEndGetWithHttpInfo(Integer dmaId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampStarttimestampEndGetValidateBeforeCall(dmaId, timestampStart, timestampEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyDmaHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of hourly astronomy for a DMA. (asynchronously)
     * Returns a contiguous range of hourly astronomical data for a Designated Market Area.
     * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call dmasDmaIdHoursTimestampStarttimestampEndGetAsync(Integer dmaId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyDmaHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = dmasDmaIdHoursTimestampStarttimestampEndGetValidateBeforeCall(dmaId, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyDmaHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
}
