/*
 * Weather Source Astronomy API
 * Daily and hourly astronomical data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source Astronomy API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */

package io.swagger.client.api;

import io.swagger.client.ApiCallback;
import io.swagger.client.ApiClient;
import io.swagger.client.ApiException;
import io.swagger.client.ApiResponse;
import io.swagger.client.Configuration;
import io.swagger.client.Pair;
import io.swagger.client.ProgressRequestBody;
import io.swagger.client.ProgressResponseBody;

import com.google.gson.reflect.TypeToken;

import java.io.IOException;


import io.swagger.client.model.AstronomyOnpointDayObj;
import io.swagger.client.model.AstronomyOnpointHourObj;
import io.swagger.client.model.ErrorObj;
import org.threeten.bp.LocalDate;
import org.threeten.bp.OffsetDateTime;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class OnpointsApi {
    private ApiClient apiClient;

    public OnpointsApi() {
        this(Configuration.getDefaultApiClient());
    }

    public OnpointsApi(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    public ApiClient getApiClient() {
        return apiClient;
    }

    public void setApiClient(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    /**
     * Build call for onpointsOnpointIdDaysDateGet
     * @param onpointId An OnPoint™ ID. (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdDaysDateGetCall(Integer onpointId, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/onpoints/{onpointId}/days/{date}"
            .replaceAll("\\{" + "onpointId" + "\\}", apiClient.escapeString(onpointId.toString()))
            .replaceAll("\\{" + "date" + "\\}", apiClient.escapeString(date.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call onpointsOnpointIdDaysDateGetValidateBeforeCall(Integer onpointId, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'onpointId' is set
        if (onpointId == null) {
            throw new ApiException("Missing the required parameter 'onpointId' when calling onpointsOnpointIdDaysDateGet(Async)");
        }
        // verify the required parameter 'date' is set
        if (date == null) {
            throw new ApiException("Missing the required parameter 'date' when calling onpointsOnpointIdDaysDateGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling onpointsOnpointIdDaysDateGet(Async)");
        }
        
        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateGetCall(onpointId, date, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A day of astronomy data for an OnPoint™ point.
     * Returns a single day of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyOnpointDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyOnpointDayObj onpointsOnpointIdDaysDateGet(Integer onpointId, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyOnpointDayObj> resp = onpointsOnpointIdDaysDateGetWithHttpInfo(onpointId, date, fields, unitScale);
        return resp.getData();
    }

    /**
     * A day of astronomy data for an OnPoint™ point.
     * Returns a single day of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyOnpointDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyOnpointDayObj> onpointsOnpointIdDaysDateGetWithHttpInfo(Integer onpointId, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateGetValidateBeforeCall(onpointId, date, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyOnpointDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A day of astronomy data for an OnPoint™ point. (asynchronously)
     * Returns a single day of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdDaysDateGetAsync(Integer onpointId, LocalDate date, List<String> fields, String unitScale, final ApiCallback<AstronomyOnpointDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateGetValidateBeforeCall(onpointId, date, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyOnpointDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for onpointsOnpointIdDaysDateStartdateEndGet
     * @param onpointId An OnPoint™ ID. (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdDaysDateStartdateEndGetCall(Integer onpointId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/onpoints/{onpointId}/days/{dateStart},{dateEnd}"
            .replaceAll("\\{" + "onpointId" + "\\}", apiClient.escapeString(onpointId.toString()))
            .replaceAll("\\{" + "dateStart" + "\\}", apiClient.escapeString(dateStart.toString()))
            .replaceAll("\\{" + "dateEnd" + "\\}", apiClient.escapeString(dateEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call onpointsOnpointIdDaysDateStartdateEndGetValidateBeforeCall(Integer onpointId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'onpointId' is set
        if (onpointId == null) {
            throw new ApiException("Missing the required parameter 'onpointId' when calling onpointsOnpointIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateStart' is set
        if (dateStart == null) {
            throw new ApiException("Missing the required parameter 'dateStart' when calling onpointsOnpointIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateEnd' is set
        if (dateEnd == null) {
            throw new ApiException("Missing the required parameter 'dateEnd' when calling onpointsOnpointIdDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling onpointsOnpointIdDaysDateStartdateEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateStartdateEndGetCall(onpointId, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of daily astronomy for an OnPoint™ point.
     * Returns a contiguous range of daily astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyOnpointDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyOnpointDayObj onpointsOnpointIdDaysDateStartdateEndGet(Integer onpointId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyOnpointDayObj> resp = onpointsOnpointIdDaysDateStartdateEndGetWithHttpInfo(onpointId, dateStart, dateEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of daily astronomy for an OnPoint™ point.
     * Returns a contiguous range of daily astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyOnpointDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyOnpointDayObj> onpointsOnpointIdDaysDateStartdateEndGetWithHttpInfo(Integer onpointId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateStartdateEndGetValidateBeforeCall(onpointId, dateStart, dateEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyOnpointDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of daily astronomy for an OnPoint™ point. (asynchronously)
     * Returns a contiguous range of daily astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdDaysDateStartdateEndGetAsync(Integer onpointId, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyOnpointDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = onpointsOnpointIdDaysDateStartdateEndGetValidateBeforeCall(onpointId, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyOnpointDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for onpointsOnpointIdHoursTimestampGet
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampGetCall(Integer onpointId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/onpoints/{onpointId}/hours/{timestamp}"
            .replaceAll("\\{" + "onpointId" + "\\}", apiClient.escapeString(onpointId.toString()))
            .replaceAll("\\{" + "timestamp" + "\\}", apiClient.escapeString(timestamp.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampGetValidateBeforeCall(Integer onpointId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'onpointId' is set
        if (onpointId == null) {
            throw new ApiException("Missing the required parameter 'onpointId' when calling onpointsOnpointIdHoursTimestampGet(Async)");
        }
        // verify the required parameter 'timestamp' is set
        if (timestamp == null) {
            throw new ApiException("Missing the required parameter 'timestamp' when calling onpointsOnpointIdHoursTimestampGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling onpointsOnpointIdHoursTimestampGet(Async)");
        }
        
        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampGetCall(onpointId, timestamp, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * An hour of astronomy data for an OnPoint™ point.
     * Returns a single hour of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyOnpointHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyOnpointHourObj onpointsOnpointIdHoursTimestampGet(Integer onpointId, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyOnpointHourObj> resp = onpointsOnpointIdHoursTimestampGetWithHttpInfo(onpointId, timestamp, fields, unitScale);
        return resp.getData();
    }

    /**
     * An hour of astronomy data for an OnPoint™ point.
     * Returns a single hour of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyOnpointHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyOnpointHourObj> onpointsOnpointIdHoursTimestampGetWithHttpInfo(Integer onpointId, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampGetValidateBeforeCall(onpointId, timestamp, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyOnpointHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * An hour of astronomy data for an OnPoint™ point. (asynchronously)
     * Returns a single hour of astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampGetAsync(Integer onpointId, OffsetDateTime timestamp, List<String> fields, String unitScale, final ApiCallback<AstronomyOnpointHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampGetValidateBeforeCall(onpointId, timestamp, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyOnpointHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for onpointsOnpointIdHoursTimestampStarttimestampEndGet
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampStarttimestampEndGetCall(Integer onpointId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/onpoints/{onpointId}/hours/{timestampStart},{timestampEnd}"
            .replaceAll("\\{" + "onpointId" + "\\}", apiClient.escapeString(onpointId.toString()))
            .replaceAll("\\{" + "timestampStart" + "\\}", apiClient.escapeString(timestampStart.toString()))
            .replaceAll("\\{" + "timestampEnd" + "\\}", apiClient.escapeString(timestampEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampStarttimestampEndGetValidateBeforeCall(Integer onpointId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'onpointId' is set
        if (onpointId == null) {
            throw new ApiException("Missing the required parameter 'onpointId' when calling onpointsOnpointIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampStart' is set
        if (timestampStart == null) {
            throw new ApiException("Missing the required parameter 'timestampStart' when calling onpointsOnpointIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampEnd' is set
        if (timestampEnd == null) {
            throw new ApiException("Missing the required parameter 'timestampEnd' when calling onpointsOnpointIdHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling onpointsOnpointIdHoursTimestampStarttimestampEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampStarttimestampEndGetCall(onpointId, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of hourly astronomy for an OnPoint™ point.
     * Returns a contiguous range of hourly astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyOnpointHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyOnpointHourObj onpointsOnpointIdHoursTimestampStarttimestampEndGet(Integer onpointId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyOnpointHourObj> resp = onpointsOnpointIdHoursTimestampStarttimestampEndGetWithHttpInfo(onpointId, timestampStart, timestampEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of hourly astronomy for an OnPoint™ point.
     * Returns a contiguous range of hourly astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyOnpointHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyOnpointHourObj> onpointsOnpointIdHoursTimestampStarttimestampEndGetWithHttpInfo(Integer onpointId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampStarttimestampEndGetValidateBeforeCall(onpointId, timestampStart, timestampEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyOnpointHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of hourly astronomy for an OnPoint™ point. (asynchronously)
     * Returns a contiguous range of hourly astronomical data for an OnPoint™ point.
     * @param onpointId An OnPoint™ ID. (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call onpointsOnpointIdHoursTimestampStarttimestampEndGetAsync(Integer onpointId, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyOnpointHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = onpointsOnpointIdHoursTimestampStarttimestampEndGetValidateBeforeCall(onpointId, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyOnpointHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
}
