/*
 * Weather Source Astronomy API
 * Daily and hourly astronomical data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source Astronomy API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */

package io.swagger.client.api;

import io.swagger.client.ApiCallback;
import io.swagger.client.ApiClient;
import io.swagger.client.ApiException;
import io.swagger.client.ApiResponse;
import io.swagger.client.Configuration;
import io.swagger.client.Pair;
import io.swagger.client.ProgressRequestBody;
import io.swagger.client.ProgressResponseBody;

import com.google.gson.reflect.TypeToken;

import java.io.IOException;


import io.swagger.client.model.AstronomyPostcodeDayObj;
import io.swagger.client.model.AstronomyPostcodeHourObj;
import io.swagger.client.model.ErrorObj;
import org.threeten.bp.LocalDate;
import org.threeten.bp.OffsetDateTime;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PostcodesApi {
    private ApiClient apiClient;

    public PostcodesApi() {
        this(Configuration.getDefaultApiClient());
    }

    public PostcodesApi(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    public ApiClient getApiClient() {
        return apiClient;
    }

    public void setApiClient(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    /**
     * Build call for postcodesPostcodecountryCodeDaysDateGet
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateGetCall(String postcode, String countryCode, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/postcodes/{postcode},{countryCode}/days/{date}"
            .replaceAll("\\{" + "postcode" + "\\}", apiClient.escapeString(postcode.toString()))
            .replaceAll("\\{" + "countryCode" + "\\}", apiClient.escapeString(countryCode.toString()))
            .replaceAll("\\{" + "date" + "\\}", apiClient.escapeString(date.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateGetValidateBeforeCall(String postcode, String countryCode, LocalDate date, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'postcode' is set
        if (postcode == null) {
            throw new ApiException("Missing the required parameter 'postcode' when calling postcodesPostcodecountryCodeDaysDateGet(Async)");
        }
        // verify the required parameter 'countryCode' is set
        if (countryCode == null) {
            throw new ApiException("Missing the required parameter 'countryCode' when calling postcodesPostcodecountryCodeDaysDateGet(Async)");
        }
        // verify the required parameter 'date' is set
        if (date == null) {
            throw new ApiException("Missing the required parameter 'date' when calling postcodesPostcodecountryCodeDaysDateGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling postcodesPostcodecountryCodeDaysDateGet(Async)");
        }
        
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateGetCall(postcode, countryCode, date, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A day of astronomy data for a postcode.
     * Returns a single day of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyPostcodeDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyPostcodeDayObj postcodesPostcodecountryCodeDaysDateGet(String postcode, String countryCode, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyPostcodeDayObj> resp = postcodesPostcodecountryCodeDaysDateGetWithHttpInfo(postcode, countryCode, date, fields, unitScale);
        return resp.getData();
    }

    /**
     * A day of astronomy data for a postcode.
     * Returns a single day of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyPostcodeDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyPostcodeDayObj> postcodesPostcodecountryCodeDaysDateGetWithHttpInfo(String postcode, String countryCode, LocalDate date, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateGetValidateBeforeCall(postcode, countryCode, date, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A day of astronomy data for a postcode. (asynchronously)
     * Returns a single day of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param date An date formatted as a RFC3339 date value. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateGetAsync(String postcode, String countryCode, LocalDate date, List<String> fields, String unitScale, final ApiCallback<AstronomyPostcodeDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateGetValidateBeforeCall(postcode, countryCode, date, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for postcodesPostcodecountryCodeDaysDateStartdateEndGet
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateStartdateEndGetCall(String postcode, String countryCode, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd}"
            .replaceAll("\\{" + "postcode" + "\\}", apiClient.escapeString(postcode.toString()))
            .replaceAll("\\{" + "countryCode" + "\\}", apiClient.escapeString(countryCode.toString()))
            .replaceAll("\\{" + "dateStart" + "\\}", apiClient.escapeString(dateStart.toString()))
            .replaceAll("\\{" + "dateEnd" + "\\}", apiClient.escapeString(dateEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateStartdateEndGetValidateBeforeCall(String postcode, String countryCode, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'postcode' is set
        if (postcode == null) {
            throw new ApiException("Missing the required parameter 'postcode' when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'countryCode' is set
        if (countryCode == null) {
            throw new ApiException("Missing the required parameter 'countryCode' when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateStart' is set
        if (dateStart == null) {
            throw new ApiException("Missing the required parameter 'dateStart' when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'dateEnd' is set
        if (dateEnd == null) {
            throw new ApiException("Missing the required parameter 'dateEnd' when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateStartdateEndGetCall(postcode, countryCode, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of daily astronomy for a postcode.
     * Returns a contiguous range of daily astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyPostcodeDayObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyPostcodeDayObj postcodesPostcodecountryCodeDaysDateStartdateEndGet(String postcode, String countryCode, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyPostcodeDayObj> resp = postcodesPostcodecountryCodeDaysDateStartdateEndGetWithHttpInfo(postcode, countryCode, dateStart, dateEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of daily astronomy for a postcode.
     * Returns a contiguous range of daily astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyPostcodeDayObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyPostcodeDayObj> postcodesPostcodecountryCodeDaysDateStartdateEndGetWithHttpInfo(String postcode, String countryCode, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateStartdateEndGetValidateBeforeCall(postcode, countryCode, dateStart, dateEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeDayObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of daily astronomy for a postcode. (asynchronously)
     * Returns a contiguous range of daily astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. (required)
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. (required)
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeDaysDateStartdateEndGetAsync(String postcode, String countryCode, LocalDate dateStart, LocalDate dateEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyPostcodeDayObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeDaysDateStartdateEndGetValidateBeforeCall(postcode, countryCode, dateStart, dateEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeDayObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for postcodesPostcodecountryCodeHoursTimestampGet
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampGetCall(String postcode, String countryCode, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/postcodes/{postcode},{countryCode}/hours/{timestamp}"
            .replaceAll("\\{" + "postcode" + "\\}", apiClient.escapeString(postcode.toString()))
            .replaceAll("\\{" + "countryCode" + "\\}", apiClient.escapeString(countryCode.toString()))
            .replaceAll("\\{" + "timestamp" + "\\}", apiClient.escapeString(timestamp.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampGetValidateBeforeCall(String postcode, String countryCode, OffsetDateTime timestamp, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'postcode' is set
        if (postcode == null) {
            throw new ApiException("Missing the required parameter 'postcode' when calling postcodesPostcodecountryCodeHoursTimestampGet(Async)");
        }
        // verify the required parameter 'countryCode' is set
        if (countryCode == null) {
            throw new ApiException("Missing the required parameter 'countryCode' when calling postcodesPostcodecountryCodeHoursTimestampGet(Async)");
        }
        // verify the required parameter 'timestamp' is set
        if (timestamp == null) {
            throw new ApiException("Missing the required parameter 'timestamp' when calling postcodesPostcodecountryCodeHoursTimestampGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling postcodesPostcodecountryCodeHoursTimestampGet(Async)");
        }
        
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampGetCall(postcode, countryCode, timestamp, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * An hour of astronomy data for a postcode.
     * Returns a single hour of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyPostcodeHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyPostcodeHourObj postcodesPostcodecountryCodeHoursTimestampGet(String postcode, String countryCode, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyPostcodeHourObj> resp = postcodesPostcodecountryCodeHoursTimestampGetWithHttpInfo(postcode, countryCode, timestamp, fields, unitScale);
        return resp.getData();
    }

    /**
     * An hour of astronomy data for a postcode.
     * Returns a single hour of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyPostcodeHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyPostcodeHourObj> postcodesPostcodecountryCodeHoursTimestampGetWithHttpInfo(String postcode, String countryCode, OffsetDateTime timestamp, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampGetValidateBeforeCall(postcode, countryCode, timestamp, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * An hour of astronomy data for a postcode. (asynchronously)
     * Returns a single hour of astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampGetAsync(String postcode, String countryCode, OffsetDateTime timestamp, List<String> fields, String unitScale, final ApiCallback<AstronomyPostcodeHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampGetValidateBeforeCall(postcode, countryCode, timestamp, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
    /**
     * Build call for postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetCall(String postcode, String countryCode, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = null;
        
        // create path and map variables
        String localVarPath = "/postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd}"
            .replaceAll("\\{" + "postcode" + "\\}", apiClient.escapeString(postcode.toString()))
            .replaceAll("\\{" + "countryCode" + "\\}", apiClient.escapeString(countryCode.toString()))
            .replaceAll("\\{" + "timestampStart" + "\\}", apiClient.escapeString(timestampStart.toString()))
            .replaceAll("\\{" + "timestampEnd" + "\\}", apiClient.escapeString(timestampEnd.toString()));

        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();
        if (fields != null)
        localVarCollectionQueryParams.addAll(apiClient.parameterToPairs("csv", "fields", fields));
        if (unitScale != null)
        localVarQueryParams.addAll(apiClient.parameterToPair("unitScale", unitScale));

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            "application/json"
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new com.squareup.okhttp.Interceptor() {
                @Override
                public com.squareup.okhttp.Response intercept(com.squareup.okhttp.Interceptor.Chain chain) throws IOException {
                    com.squareup.okhttp.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] { "ApiKeyAuth" };
        return apiClient.buildCall(localVarPath, "GET", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }
    
    @SuppressWarnings("rawtypes")
    private com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetValidateBeforeCall(String postcode, String countryCode, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        // verify the required parameter 'postcode' is set
        if (postcode == null) {
            throw new ApiException("Missing the required parameter 'postcode' when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'countryCode' is set
        if (countryCode == null) {
            throw new ApiException("Missing the required parameter 'countryCode' when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampStart' is set
        if (timestampStart == null) {
            throw new ApiException("Missing the required parameter 'timestampStart' when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'timestampEnd' is set
        if (timestampEnd == null) {
            throw new ApiException("Missing the required parameter 'timestampEnd' when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(Async)");
        }
        // verify the required parameter 'fields' is set
        if (fields == null) {
            throw new ApiException("Missing the required parameter 'fields' when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(Async)");
        }
        
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetCall(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        return call;

        
        
        
        
    }

    /**
     * A contiguous range of hourly astronomy for a postcode.
     * Returns a contiguous range of hourly astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return AstronomyPostcodeHourObj
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public AstronomyPostcodeHourObj postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(String postcode, String countryCode, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        ApiResponse<AstronomyPostcodeHourObj> resp = postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetWithHttpInfo(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale);
        return resp.getData();
    }

    /**
     * A contiguous range of hourly astronomy for a postcode.
     * Returns a contiguous range of hourly astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ApiResponse&lt;AstronomyPostcodeHourObj&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<AstronomyPostcodeHourObj> postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetWithHttpInfo(String postcode, String countryCode, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale) throws ApiException {
        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetValidateBeforeCall(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale, null, null);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeHourObj>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * A contiguous range of hourly astronomy for a postcode. (asynchronously)
     * Returns a contiguous range of hourly astronomical data for a postcode.
     * @param postcode A postcode string. (required)
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). (required)
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. (required)
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. (required)
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allMoon *(moon related fields)* * allSun *(sun related fields)* * popular *(sunrise, sunset, moonrise, moonset, and moonPhase)*  (required)
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public com.squareup.okhttp.Call postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetAsync(String postcode, String countryCode, OffsetDateTime timestampStart, OffsetDateTime timestampEnd, List<String> fields, String unitScale, final ApiCallback<AstronomyPostcodeHourObj> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        com.squareup.okhttp.Call call = postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetValidateBeforeCall(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<AstronomyPostcodeHourObj>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
}
